﻿using System;
using System.Collections;
using System.Linq;
using System.Reflection;
using System.Text;

using NHibernate.Cfg.MappingSchema;
using NHibernate.Linq;
using NHibernate.Mapping.ByCode;

using NUnit.Framework;

namespace NHibernate.Test.NHSpecificTest.NH3121
{
    [TestFixture]
    public class Fixture : TestCase
    {
        [Test]
        public void ShouldReturnCorrectBlobSize()
        {
            var conf = TestConfigurationHelper.GetDefaultConfiguration();

            AddMappings(conf);

            BuildSessionFactory();

            var templateSize = 756857;
            var reportSize = 17158;

            ISession session = OpenSession();

            var templateImage = new Byte[templateSize];
            var reportImage = new Byte[reportSize];

            var random = new Random();
            random.NextBytes(templateImage);
            random.NextBytes(reportImage);

            var template = new Template
            {
                Name = "Template1",
                Image = templateImage
            };

            session.Save(template);
            session.Flush();

            var report = new Report
            {
                Name = "report1",
                Image = reportImage,
                Template = template
            };

            session.Save(report);
            session.Flush();

            var firstReport = (from t in session.Query<Report>()
                               select new
                               {
                                   reportImage = t.Image,
                                   templateImage = t.Template.Image
                               }
                          ).First();

            session.Close();

            Assert.AreEqual(reportSize, firstReport.reportImage.Length);
            Assert.AreEqual(templateSize, firstReport.templateImage.Length);
        }

        protected override IList Mappings
        {
            get { return new string[0]; }
        }

        protected override string MappingsAssembly
        {
            get { return null; }
        }

        protected override void AddMappings(Cfg.Configuration configuration)
        {
            configuration.AddDeserializedMapping(GetMappings(), "TestDomain");
        }

        private HbmMapping GetMappings()
        {
            var mapper = new ModelMapper();

            mapper.BeforeMapClass += (mi, t, map) => map.Id(x => x.Generator(Generators.GuidComb));

            mapper.Class<Template>(rc =>
            {
                rc.Id(p => p.Id);
                rc.Property(p => p.Name);
                rc.Property(p => p.Image
			         m => m.Column(c => c.SqlType("varbinary(max)")));
                rc.Bag(p => p.Reports, m =>
                {
                    m.Key(km => { km.Column(cm => cm.Name("TemplateId")); });
                    m.Inverse(true);
                    m.Cascade(Mapping.ByCode.Cascade.Persist);
                }, rel => rel.OneToMany());
            });

            mapper.Class<Report>(rc =>
            {
                rc.Id(p => p.Id);
                rc.Property(p => p.Name);
                rc.Property(p => p.Image
			         m => m.Column(c => c.SqlType("varbinary(max)")));
                rc.ManyToOne<Template>(p => p.Template, m => { m.Column("TemplateId"); });
            });

            return mapper.CompileMappingForAllExplicitlyAddedEntities();
        }
    }
}
